﻿using Cop.Borovkov.Var3.Exceptions;

namespace Cop.Borovkov.Var3.Components
{
    public partial class CustomNumericInputField : UserControl
    {
        private event EventHandler? _numericInputChanged;

        private event Action<Exception>? _errorOccured;

        /// <summary>
        /// </summary>
        public CustomNumericInputField()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Событие возникающие при изменении поля ввода
        /// </summary>
        public event EventHandler NumericInputChanged
        {
            add => _numericInputChanged += value;
            remove => _numericInputChanged -= value;
        }

        /// <summary>
        /// Обработка ошибок
        /// </summary>
        public event Action<Exception> AnErrorOccurred
        {
            add => _errorOccured += value;
            remove => _errorOccured -= value;
        }

        /// <summary>
        /// Значения поля
        /// </summary>
        public int? Value
        {
            get
            {
                if (isNullcheckBox.Checked)
                {
                    return null;
                }

                if (int.TryParse(inputField.Text, out var value))
                {
                    return value;
                }

                throw new InvalidNumericInputValueException();
            }
            set
            {
                if (value is null)
                {
                    isNullcheckBox.Checked = true;
                }
                else
                {
                    isNullcheckBox.Checked = false;
                    inputField.Text = value.ToString();
                }
            }
        }

        private void IsNullcheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (isNullcheckBox.Checked)
            {
                inputField.Text = string.Empty;
                inputField.Enabled = false;
            }
            else
            {
                inputField.Enabled = true;
            }

            try
            {
                _numericInputChanged?.Invoke(sender, e);
            }
            catch (Exception ex)
            {
                _errorOccured?.Invoke(ex);
            }
        }

        private void InputField_DataContextChanged(object sender, EventArgs e)
        {
            try
            {
                _numericInputChanged?.Invoke(sender, e);
            }
            catch (Exception ex)
            {
                _errorOccured?.Invoke(ex);
            }
        }
    }
}
